import * as React from "react";
import * as PropTypes from "prop-types";
import { BaseRootQuery } from "bicycle/typed-helpers/query";
import BicycleClient, { QueryCacheResult } from "bicycle/client";
import ErrorResult from "bicycle/types/ErrorResult";
import DeepPartial from "../DeepPartial";
export { PropTypes };
export interface Status<QueryResult> {
    result: DeepPartial<QueryResult>;
    loaded: boolean;
    errors: ReadonlyArray<string>;
    errorDetails: ReadonlyArray<ErrorResult>;
    previousElement: React.ReactNode;
    loadingDuration: number;
}
export declare type RenderLoaded<QueryResult> = (result: QueryResult, client: BicycleClient, status: Status<QueryResult>) => React.ReactNode;
export declare type RenderMaybeLoaded<QueryResult> = (result: DeepPartial<QueryResult>, client: BicycleClient, status: Status<QueryResult>) => React.ReactNode;
export declare type RenderLoading<QueryResult> = (loadingDuration: number, client: BicycleClient, status: Status<QueryResult>) => React.ReactNode;
export declare type RenderErrors<QueryResult> = (errors: ReadonlyArray<string>, client: BicycleClient, status: Status<QueryResult>) => React.ReactNode;
export interface Props<QueryResult> {
    query: BaseRootQuery<QueryResult>;
    children: RenderLoaded<QueryResult>;
    renderLoading?: boolean | RenderLoading<QueryResult>;
    renderErrors?: boolean | RenderErrors<QueryResult>;
}
export interface State<QueryResult> extends QueryCacheResult<QueryResult> {
    loadingDuration: number;
}
export default class Connect<QueryResult> extends React.Component<Props<QueryResult>, State<QueryResult>> {
    context: {
        bicycleClient: BicycleClient;
    };
    static contextTypes: {
        bicycleClient: PropTypes.Requireable<PropTypes.InferProps<{
            subscribe: PropTypes.Validator<(...args: any[]) => any>;
            update: PropTypes.Validator<(...args: any[]) => any>;
            queryCache: PropTypes.Validator<(...args: any[]) => any>;
        }>>;
    };
    private _client;
    private _query;
    private _subscription;
    private _previousElement;
    _startLoadingTime: number | null;
    constructor(props: Props<QueryResult>, context: any);
    componentDidMount(): void;
    componentWillReceiveProps(nextProps: Props<QueryResult>): void;
    componentWillUnmount(): void;
    _onUpdate: (result: QueryResult, loaded: boolean, errors: ReadonlyArray<string>, errorDetails: ReadonlyArray<ErrorResult>) => void;
    _render(): {} | null | undefined;
    render(): any;
}
export declare function query<QueryResult>(query: BaseRootQuery<QueryResult>, children: RenderLoaded<QueryResult>): React.ReactElement<any>;
export declare function query<QueryResult>(query: BaseRootQuery<QueryResult>, children: RenderLoaded<QueryResult>, options: {
    renderLoading?: RenderLoading<QueryResult>;
    renderErrors?: RenderErrors<QueryResult>;
}): React.ReactElement<any>;
export declare function query<QueryResult>(query: BaseRootQuery<QueryResult>, children: RenderMaybeLoaded<QueryResult>, options: {
    renderLoading?: boolean | RenderLoading<QueryResult>;
    renderErrors?: boolean | RenderErrors<QueryResult>;
}): React.ReactElement<any>;
